from django import template
from django.conf import settings
from django.utils.formats import number_format
from django.utils.translation import ugettext as _

register = template.Library()


@register.simple_tag(takes_context=True)
def querystring(context, **kwargs):
    args = context['request'].GET.copy()
    for k, v in kwargs.items():
        args[k] = v
    return args.urlencode()


SORT_FIELD = getattr(settings, 'SORT_FIELD', 'o')

SORT_FIELD = getattr(settings, 'SORT_FIELD', 'o')


SORT_ICON = getattr(settings, 'SORT_ICON', '<i class="fa fa-sort"></i>')
SORT_ASC_ICON = getattr(settings, 'SORT_ASC_ICON', '<i class="fa fa-sort-amount-asc"></i>')
SORT_DESC_ICON = getattr(settings, 'SORT_DESC_ICON', '<i class="fa fa-sort-amount-desc"></i>')

SORTANCHOR_TEMPLATE = '<a href="%(url)s">%(text)s</a>'

CRISPY_TEMPLATE_PACK = getattr(settings, 'CRISPY_TEMPLATE_PACK')
if CRISPY_TEMPLATE_PACK == 'bootstrap4':
    SORTANCHOR_TEMPLATE = '<a class="text-decoration-none" href="%(url)s">%(text)s <small>%(icon)s</small></a>'

if CRISPY_TEMPLATE_PACK == 'bootstrap5':
    SORTANCHOR_TEMPLATE = '<a class="text-decoration-none" href="%(url)s">%(text)s <small>%(icon)s</small></a>'


SORTANCHOR_TEMPLATE = getattr(settings, 'SORANCHOR_TEMPLATE', SORTANCHOR_TEMPLATE)


def sortanchor(parser, token):
    """
    Parses a tag that's supposed to be in this format
        '{% sortanchor field title [default] %}'
    Title may be a "string", _("trans string"), or variable
    """
    bits = [b for b in token.split_contents()]
    if len(bits) < 2:
        raise template.TemplateSyntaxError(
                        "anchor tag takes at least 1 argument.")

    try:
        title = bits[2]
    except IndexError:
        title = bits[1].capitalize()

    if len(bits) > 3:
        default = bits[3]
    else:
        default = False

    return SortAnchorNode(bits[1].strip(), title, default)


class SortAnchorNode(template.Node):
    """
    Renders an <a> HTML tag with a link which href attribute
    includes the field on which we sort and the direction.
    and adds an up or down arrow if the field is the one
    currently being sorted on.

    Eg.
        {% sortanchor name Name %} generates
        <a href="/the/current/path/?sort=name" title="Name">Name</a>

    """
    def __init__(self, field, title, is_default):
        self.field = field
        self.title = title
        self.is_default = is_default

    def render(self, context):

        title = self.title

        if title[0] in ('"', "'"):
            if title[0] == title[-1]:
                title = title[1:-1]
            else:
                raise template.TemplateSyntaxError(
                        'sortanchor tag title '
                        'must be a "string", _("trans string"), or variable'
                        )
        elif title.startswith('_("') or title.startswith("_('"):
            title = _(title[3:-2])
        else:
            #  Title is var
            title = context[title]

        request = context['request']
        getvars = request.GET.copy()

        currentsort = ''
        sortby = self.field
        icon = SORT_ICON

        def invert_order(fields):
            res = []
            for field in fields.split(","):
                if field.startswith("-"):
                    res.append(field[1:])
                else:
                    res.append("-" + field)
            return ",".join(res)

        if SORT_FIELD in getvars:
            currentsort = getvars[SORT_FIELD]
        else:
            if self.is_default:
                if self.is_default == '-':
                    currentsort = invert_order(self.field)
                else:
                    currentsort = self.field

        if currentsort == self.field:
            sortby = invert_order(self.field)
            icon = SORT_ASC_ICON

        if invert_order(currentsort) == self.field:
            icon = SORT_DESC_ICON

        getvars[SORT_FIELD] = sortby

        url = '%s?%s' % (request.path, getvars.urlencode())
        return SORTANCHOR_TEMPLATE % {"url": url, "text": title, "icon": icon}


sortanchor = register.tag(sortanchor)


@register.filter
def money_format(num):
    if num is None:
        return ""
    if isinstance(num, str):
        return num
    return number_format(num, decimal_pos=2, force_grouping=True)




from django import forms


class UpperCharField(forms.CharField):

    def clean(self, value):
        value = super(UpperCharField, self).clean(value)
        return value.upper()

    def widget_attrs(self, widget):
        attrs = super(UpperCharField, self).widget_attrs(widget)
        if 'class' in attrs:
            attrs['class'] += ' uppercase'
        else:
            attrs['class'] = 'uppercase'
        return attrs



import re

from django.conf import settings
from django.contrib.auth.decorators import login_required
from django.utils import timezone
from django.utils import translation

try:
    LANGUAGE_SESSION_KEY = translation.LANGUAGE_SESSION_KEY
except AttributeError:
    LANGUAGE_SESSION_KEY = '_language'

LANGUAGE_COOKIE_AGE = getattr(settings, "LANGUAGE_COOKIE_AGE", settings.SESSION_COOKIE_AGE)
LANGUAGE_COOKIE_PATH = getattr(settings, "LANGUAGE_COOKIE_PATH", settings.SESSION_COOKIE_PATH)
LANGUAGE_COOKIE_DOMAIN = getattr(settings, "ANGUAGE_COOKIE_DOMAIN", settings.SESSION_COOKIE_DOMAIN)


## From django 1.10
class MiddlewareMixin(object):
    def __init__(self, get_response=None):
        self.get_response = get_response
        super(MiddlewareMixin, self).__init__()

    def __call__(self, request):
        response = None
        if hasattr(self, 'process_request'):
            response = self.process_request(request)
        if not response:
            response = self.get_response(request)
        if hasattr(self, 'process_response'):
            response = self.process_response(request, response)
        return response


## from digital.models import Profile
class TimezoneMiddleware(MiddlewareMixin):
    def process_request(self, request):
        tz = None
        if request.user.is_authenticated:
            try:
                tz = request.user.profile.timezone
            except Profile.DoesNotExist:
                pass
        if tz:
            timezone.activate(tz)
        else:
            timezone.deactivate()


#
# From https://djangosnippets.org/snippets/1220/
#
class RequireLoginMiddleware(MiddlewareMixin):
    """
    Middleware component that wraps the login_required decorator around
    matching URL patterns. To use, add the class to MIDDLEWARE_CLASSES and
    define LOGIN_REQUIRED_URLS and LOGIN_REQUIRED_URLS_EXCEPTIONS in your
    settings.py. For example:
    ------
    LOGIN_REQUIRED_URLS = (
        r'/topsecret/(.*)$',
    )
    LOGIN_REQUIRED_URLS_EXCEPTIONS = (
        r'/topsecret/login(.*)$',
        r'/topsecret/logout(.*)$',
    )
    ------
    LOGIN_REQUIRED_URLS is where you define URL patterns; each pattern must
    be a valid regex.

    LOGIN_REQUIRED_URLS_EXCEPTIONS is, conversely, where you explicitly
    define any exceptions (like login and logout URLs).
    """

    def __init__(self, get_response=None):
        super(RequireLoginMiddleware, self).__init__(get_response)

        self.required = tuple(re.compile(url) for url in
                                      settings.LOGIN_REQUIRED_URLS)
        self.exceptions = tuple(re.compile(url) for url in
                                      settings.LOGIN_REQUIRED_URLS_EXCEPTIONS)

    def process_view(self, request, view_func, view_args, view_kwargs):
        # No need to process URLs if user already logged in
        if request.user.is_authenticated:
            return None

        # An exception match should immediately return None
        for url in self.exceptions:
            if url.match(request.path_info):
                return None

        # Requests matching a restricted URL pattern are returned
        # wrapped with the login_required decorator
        for url in self.required:
            if url.match(request.path_info):
                return login_required(view_func)(request,
                                        *view_args, **view_kwargs)

        # Explicitly return None for all non-matching requests
        return None


class GetParamLocaleMiddleware(MiddlewareMixin):
    """
    This Middleware sets the language from a GET param.
    """

    def get_lang_param(self, request):
        if request.method == 'GET' and request.GET.get('lang'):
            language = request.GET['lang']
            for lang in settings.LANGUAGES:
                if lang[0] == language:
                    return language
        return None

    def process_request(self, request):
        language = self.get_lang_param(request)
        if language:
            translation.activate(language)
            request.LANGUAGE_CODE = translation.get_language()

    def process_response(self, request, response):
        language = self.get_lang_param(request)
        if language:
            if hasattr(request, 'session'):
                request.session[LANGUAGE_SESSION_KEY] = language
            response.set_cookie(settings.LANGUAGE_COOKIE_NAME, language,
                                max_age=LANGUAGE_COOKIE_AGE,
                                path=LANGUAGE_COOKIE_PATH,
                                domain=LANGUAGE_COOKIE_DOMAIN)
        return response



from django.contrib.auth.models import User
from django.contrib.auth.forms import AuthenticationForm
from django.utils.translation import ugettext_lazy as _
from django.core.exceptions import MultipleObjectsReturned
from django import forms


class EmailAuthBackend(object):
    """
    Email Authentication Backend

    Allows a user to sign in using an email/password pair rather than
    a username/password pair.
    """

    def authenticate(self, username=None, password=None):
        """ Authenticate a user based on email address as the user name. """
        try:
            user = User.objects.get(email=username)
            if user.check_password(password):
                return user
        except (User.DoesNotExist, MultipleObjectsReturned):
            return None

    def get_user(self, user_id):
        """ Get a User object from the user_id. """
        try:
            return User.objects.get(pk=user_id)
        except User.DoesNotExist:
            return None


class IdAuthBackend(object):
    """
    Id Authentication Backend

    Allows a user to sign in using an id/password pair rather than
    a username/password pair.
    """

    def authenticate(self, username=None, password=None):
        """ Authenticate a user based on email address as the user name. """
        try:
            user_id = int(username)
        except ValueError:
            return None

        try:
            user = User.objects.get(pk=user_id)
            if user.check_password(password):
                return user
        except User.DoesNotExist:
            return None

    def get_user(self, user_id):
        """ Get a User object from the user_id. """
        try:
            return User.objects.get(pk=user_id)
        except (User.DoesNotExist, MultipleObjectsReturned):
            return None


class CustomAuthenticationForm(AuthenticationForm):
    username = forms.CharField(label=_("Username or email address"),
                max_length=254)



#!/usr/bin/env python
import os
from distutils.core import setup

__author__ = u'Ferran Pegueroles'
__copyright__ = u'Copyright 2014, Ferran Pegueroles'
__credits__ = [u'Ferran Pegueroles']


__license__ = 'BSDD'
__version__ = '0.1.15'
__email__ = 'ferran@pegueroles.com'


long_description = open(os.path.join(os.path.dirname(__file__),
                    'README.rst')).read()


setup(
    name='djgeneric',
    version=__version__,
    url='http://bitbucket.org/ferranp/djgeneric',
    author=__author__,
    author_email=__email__,
    license='GPL',
    packages=['djgeneric', 'djgeneric.templatetags'],
    description='Generic utilities for django',
    long_description=long_description,
    classifiers=['Development Status :: 4 - Beta',
                 'Environment :: Web Environment',
                 'Framework :: Django',
                 'Intended Audience :: Developers',
                 'License :: OSI Approved :: BSD License',
                 'Topic :: Internet :: WWW/HTTP :: Dynamic Content']
)




from django.conf import settings


def google_analytics(request):
    """
    Use the variables returned in this function to
    render your Google Analytics tracking code template.

    http://www.nomadblue.com/blog/django/google-analytics-tracking-code-into-django-project/

    """
    ga_prop_id = getattr(settings, 'GOOGLE_ANALYTICS_PROPERTY_ID', False)
    ga_domain = getattr(settings, 'GOOGLE_ANALYTICS_DOMAIN', False)
    if not settings.DEBUG and ga_prop_id and ga_domain:
        return {
            'GOOGLE_ANALYTICS_PROPERTY_ID': ga_prop_id,
            'GOOGLE_ANALYTICS_DOMAIN': ga_domain,
        }
    return {}
